import React from "react";

import "../init";
import PropTypes from "prop-types";
import styling from "@devcake/styling";
import { FormElement } from "../formElement";
import { useStyle } from "../../Scope";
import { Color, ColorPropType } from "../../types";

styling.setDefault("components.checkbox.container", "items-center");
styling.setDefault("components.checkbox.input", "border-gray-300");
styling.setDefault("components.checkbox.label.left", "mr-2");
styling.setDefault("components.checkbox.label.right", "ml-2");

export type CheckboxProps = FormElement<boolean> & {
  color: Color;
  labelPosition?: "left" | "right";
  labelClassName?: string;
};

export const Checkbox = ({
                           className,
                           color,
                           disabled,
                           label,
                           labelPosition,
                           labelClassName,
                           onChange,
                           value
                         }: CheckboxProps) => {
  const [container, input, labelLeft, labelRight] = useStyle(["container", "input", "label.left", "label.right"], "components.checkbox");
  const [checkboxValue, setCheckboxValue] = React.useState<boolean>(value);

  React.useEffect(() => {
    setCheckboxValue(value);
  }, [value]);

  return (
    <div className={`flex flex-row ${container}`}>
      {label && labelPosition === "left" && (
        <label
          className={`${labelClassName ? labelClassName : ""} ${labelLeft}`}
        >
          {label}
        </label>
      )}
      <input
        type="checkbox"
        className={`${input} focus:ring-${color}-500 text-${color}-600 hover:bg-${color}-300 ${className ? className : ""}`}
        onChange={(evt) => {
          const { checked } = evt.target;
          setCheckboxValue(checked);

          if (onChange) onChange(checked);
        }}
        disabled={disabled}
        checked={checkboxValue}
      />
      {label && labelPosition === "right" && (
        <label
          className={`${labelClassName} ${labelRight}`}
        >
          {label}
        </label>
      )}
    </div>
  );
};

Checkbox.defaultProps = {
  color: "blue",
  labelPosition: "right",
  disabled: false,
};

Checkbox.propTypes = {
  color: ColorPropType,
  className: PropTypes.string,
  label: PropTypes.string,
  labelPosition: PropTypes.oneOf(["left", "right"]),
  labelClassName: PropTypes.string,
  onChange: PropTypes.func,
  disabled: PropTypes.bool,
};
Checkbox.defaultProps = {
  onChange: () => null
};
